/*
 * Copyright (c) 2003-2010, John Wiegley.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *
 * - Neither the name of New Artisans LLC nor the names of its
 *   contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <system.hh>

#include "utils.h"

namespace ledger {

std::streamsize straccbuf::xsputn(const char * s, std::streamsize num)
{
  if (index == 0) {
    // The first item received is the format string
    str = std::string(s, static_cast<std::string::size_type>(num));
  }
  else {
    std::ostringstream buf;

    // Every item thereafter is an argument that substitutes for %# in the
    // format string
    bool matched = false;
    for (const char * p = str.c_str(); *p; p++) {
      if (*p == '%') {
        const char * q = p + 1;
        if (*q && *q != '%' && std::isdigit(*q) &&
            std::string::size_type(*q - '0') == index) {
          p++;
          buf << std::string(s, static_cast<std::string::size_type>(num));
          matched = true;
        } else {
          buf << *p;
        }
      } else {
        buf << *p;
      }
    }
    if (! matched)
      buf << std::string(s, static_cast<std::string::size_type>(num));

    str = buf.str();
  }
  index++;
  return num;
}

} // namespace ledger
